@@ -5,7 +5,6 @@
  */
 #include "dailyweatherforecasttest.h"
 #include <kweathercore/hourlyweatherforecast.h>
-#include <kweathercore/sunrise.h>
 
 using namespace KWeatherCore;
 
@@ -22,7 +21,7 @@ void DailyForecastTest::testHourlyMerge()
         h.setWeatherIcon(QStringLiteral("sunny"));
         h.setTemperature(25);
         h.setPressure(100);
-        h.setWindDirection(WindDirection::E);
+        h.setWindDirectionDegree(90.0);
         h.setFog(0.2);
         h.setUvIndex(0.8);
         h.setWindSpeed(5);
@@ -49,12 +48,4 @@ void DailyForecastTest::testJson()
 
     QCOMPARE(d3.toJson(), d2.toJson());
 }
-void DailyForecastTest::testSunrise()
-{
-    auto date = QDateTime::currentDateTime();
-    Sunrise s;
-    s.setSunRise(date);
-    d2.setSunrise(s);
-    QCOMPARE(d2.sunrise().sunRise(), date);
-}
 QTEST_MAIN(DailyForecastTest)
