@@ -15,13 +15,13 @@ public:
     QString title;
     QString summary;
     QString area;
-    AlertInfo::Urgency urgency = AlertInfo::Urgency::Unknown;
-    AlertInfo::Severity severity = AlertInfo::Severity::Unknown;
-    AlertInfo::Certainty certainty = AlertInfo::Certainty::Unknown;
+    CAPAlertInfo::Urgency urgency = CAPAlertInfo::Urgency::UnknownUrgency;
+    CAPAlertInfo::Severity severity = CAPAlertInfo::Severity::UnknownSeverity;
+    CAPAlertInfo::Certainty certainty = CAPAlertInfo::Certainty::UnknownCertainty;
     QDateTime date;
     QUrl CAPUrl;
     AreaCodeVec areaCodes;
-    Polygon polygon;
+    CAPPolygon polygon;
 };
 
 AlertFeedEntry::AlertFeedEntry()
@@ -72,7 +72,7 @@ const AreaCodeVec &AlertFeedEntry::areaCodes() const
 {
     return d->areaCodes;
 }
-const Polygon &AlertFeedEntry::polygon() const
+const CAPPolygon &AlertFeedEntry::polygon() const
 {
     return d->polygon;
 }
@@ -89,15 +89,15 @@ void AlertFeedEntry::setArea(const QString &area)
 {
     d->area = area;
 }
-void AlertFeedEntry::setUrgency(AlertInfo::Urgency urgency)
+void AlertFeedEntry::setUrgency(CAPAlertInfo::Urgency urgency)
 {
     d->urgency = urgency;
 }
-void AlertFeedEntry::setCertainty(AlertInfo::Certainty certainty)
+void AlertFeedEntry::setCertainty(CAPAlertInfo::Certainty certainty)
 {
     d->certainty = certainty;
 }
-void AlertFeedEntry::setSeverity(AlertInfo::Severity severity)
+void AlertFeedEntry::setSeverity(CAPAlertInfo::Severity severity)
 {
     d->severity = severity;
 }
@@ -113,15 +113,11 @@ void AlertFeedEntry::setAreaCodes(const AreaCodeVec &areaCodes)
 {
     d->areaCodes = areaCodes;
 }
-void AlertFeedEntry::setPolygon(const Polygon &polygon)
-{
-    d->polygon = polygon;
-}
 void AlertFeedEntry::setAreaCodes(AreaCodeVec &&areaCodes)
 {
     d->areaCodes = std::move(areaCodes);
 }
-void AlertFeedEntry::setPolygon(Polygon &&polygon)
+void AlertFeedEntry::setPolygon(CAPPolygon &&polygon)
 {
     d->polygon = std::move(polygon);
 }
