@@ -5,6 +5,11 @@
  */
 
 #include "capparser.h"
+#include "capalertinfo.h"
+#include "capalertmessage.h"
+#include "caparea.h"
+#include "capnamedvalue.h"
+#include "capreference.h"
 #include "kweathercore_p.h"
 #include <KLocalizedString>
 #include <QDateTime>
@@ -34,22 +39,22 @@ static std::optional<EnumT> stringToValue(const QStringT &s, const MapEntry<Enum
 }
 
 // ### important: keep all the following tables sorted by name!
-static constexpr const MapEntry<AlertInfo::Category> category_map[] = {
-    {"CBRNE", AlertInfo::Category::CBRNE},
-    {"Env", AlertInfo::Category::Env},
-    {"Fire", AlertInfo::Category::Fire},
-    {"Geo", AlertInfo::Category::Geo},
-    {"Health", AlertInfo::Category::Health},
-    {"Infra", AlertInfo::Category::Infra},
-    {"Met", AlertInfo::Category::Met},
-    {"Other", AlertInfo::Category::Other},
-    {"Rescue", AlertInfo::Category::Rescue},
-    {"Safety", AlertInfo::Category::Safety},
-    {"Security", AlertInfo::Category::Security},
-    {"Transport", AlertInfo::Category::Transport},
+static constexpr const MapEntry<CAPAlertInfo::Category> category_map[] = {
+    {"CBRNE", CAPAlertInfo::Category::CBRNE},
+    {"Env", CAPAlertInfo::Category::Environmental},
+    {"Fire", CAPAlertInfo::Category::Fire},
+    {"Geo", CAPAlertInfo::Category::Geophysical},
+    {"Health", CAPAlertInfo::Category::Health},
+    {"Infra", CAPAlertInfo::Category::Infrastructure},
+    {"Met", CAPAlertInfo::Category::Meteorological},
+    {"Other", CAPAlertInfo::Category::Other},
+    {"Rescue", CAPAlertInfo::Category::Rescue},
+    {"Safety", CAPAlertInfo::Category::Safety},
+    {"Security", CAPAlertInfo::Category::Security},
+    {"Transport", CAPAlertInfo::Category::Transport},
 };
 
-enum class Tags { ALERT, IDENTIFIER, SENDER, SENT_TIME, STATUS, MSG_TYPE, SCOPE, NOTE, INFO };
+enum class Tags { ALERT, IDENTIFIER, SENDER, SENT_TIME, STATUS, MSG_TYPE, SCOPE, NOTE, INFO, REFERENCES };
 
 static constexpr const MapEntry<Tags> tag_map[] = {
     {"alert", Tags::ALERT},
@@ -57,6 +62,7 @@ static constexpr const MapEntry<Tags> tag_map[] = {
     {"info", Tags::INFO},
     {"msgType", Tags::MSG_TYPE},
     {"note", Tags::NOTE},
+    {"references", Tags::REFERENCES},
     {"scope", Tags::SCOPE},
     {"sender", Tags::SENDER},
     {"sent", Tags::SENT_TIME},
@@ -67,6 +73,7 @@ enum class InfoTags {
     HEADLINE,
     DESCRIPTION,
     EVENT,
+    EVENTCODE,
     EFFECTIVE_TIME,
     ONSET_TIME,
     EXPIRE_TIME,
@@ -78,46 +85,66 @@ enum class InfoTags {
     PARAMETER,
     AREA,
     SENDERNAME,
-    LANGUAGE
+    LANGUAGE,
+    RESPONSETYPE,
+    CONTACT,
+    WEB,
 };
 
 static constexpr const MapEntry<InfoTags> info_tag_map[] = {
     {"area", InfoTags::AREA},
     {"category", InfoTags::CATEGORY},
     {"certainty", InfoTags::CERTAINITY},
+    {"contact", InfoTags::CONTACT},
     {"description", InfoTags::DESCRIPTION},
     {"effective", InfoTags::EFFECTIVE_TIME},
     {"event", InfoTags::EVENT},
+    {"eventCode", InfoTags::EVENTCODE},
     {"expires", InfoTags::EXPIRE_TIME},
     {"headline", InfoTags::HEADLINE},
     {"instruction", InfoTags::INSTRUCTION},
     {"language", InfoTags::LANGUAGE},
     {"onset", InfoTags::ONSET_TIME},
+    {"parameter", InfoTags::PARAMETER},
+    {"responseType", InfoTags::RESPONSETYPE},
     {"senderName", InfoTags::SENDERNAME},
     {"severity", InfoTags::SEVERITY},
     {"urgency", InfoTags::URGENCY},
+    {"web", InfoTags::WEB},
 };
 
-static constexpr const MapEntry<AlertEntry::Status> status_map[] = {
-    {"Actual", AlertEntry::Status::Actual},
-    {"Draft", AlertEntry::Status::Draft},
-    {"Excercise", AlertEntry::Status::Exercise},
-    {"System", AlertEntry::Status::System},
-    {"Test", AlertEntry::Status::Test},
+static constexpr const MapEntry<CAPAlertMessage::Status> status_map[] = {
+    {"Actual", CAPAlertMessage::Status::Actual},
+    {"Draft", CAPAlertMessage::Status::Draft},
+    {"Excercise", CAPAlertMessage::Status::Exercise},
+    {"System", CAPAlertMessage::Status::System},
+    {"Test", CAPAlertMessage::Status::Test},
 };
 
-static constexpr const MapEntry<AlertEntry::MsgType> msgtype_map[] = {
-    {"Ack", AlertEntry::MsgType::Ack},
-    {"Alert", AlertEntry::MsgType::Alert},
-    {"Cancel", AlertEntry::MsgType::Cancel},
-    {"Error", AlertEntry::MsgType::Error},
-    {"Update", AlertEntry::MsgType::Update},
+static constexpr const MapEntry<CAPAlertMessage::MessageType> msgtype_map[] = {
+    {"Ack", CAPAlertMessage::MessageType::Acknowledge},
+    {"Alert", CAPAlertMessage::MessageType::Alert},
+    {"Cancel", CAPAlertMessage::MessageType::Cancel},
+    {"Error", CAPAlertMessage::MessageType::Error},
+    {"Update", CAPAlertMessage::MessageType::Update},
 };
 
-static constexpr const MapEntry<AlertEntry::Scope> scope_map[] = {
-    {"Private", AlertEntry::Scope::Private},
-    {"Public", AlertEntry::Scope::Public},
-    {"Restricted", AlertEntry::Scope::Restricted},
+static constexpr const MapEntry<CAPAlertMessage::Scope> scope_map[] = {
+    {"Private", CAPAlertMessage::Scope::Private},
+    {"Public", CAPAlertMessage::Scope::Public},
+    {"Restricted", CAPAlertMessage::Scope::Restricted},
+};
+
+static constexpr const MapEntry<CAPAlertInfo::ResponseType> response_type_map[] = {
+    {"AllClear", CAPAlertInfo::ResponseType::AllClear},
+    {"Assess", CAPAlertInfo::ResponseType::Assess},
+    {"Avoid", CAPAlertInfo::ResponseType::Avoid},
+    {"Evacuate", CAPAlertInfo::ResponseType::Evacuate},
+    {"Execute", CAPAlertInfo::ResponseType::Execute},
+    {"Monitor", CAPAlertInfo::ResponseType::Monitor},
+    {"None", CAPAlertInfo::ResponseType::None},
+    {"Prepare", CAPAlertInfo::ResponseType::Prepare},
+    {"Shelter", CAPAlertInfo::ResponseType::Shelter},
 };
 
 CAPParser::CAPParser(const QByteArray &data)
@@ -128,7 +155,6 @@ CAPParser::CAPParser(const QByteArray &data)
         while (m_xml.readNextStartElement()) {
             if (m_xml.name() == QStringLiteral("alert")) {
                 flag = true;
-                qDebug() << "valid cap file";
                 break;
             }
         }
@@ -137,15 +163,10 @@ CAPParser::CAPParser(const QByteArray &data)
         }
     }
 }
-void CAPParser::setData(const QByteArray &data)
-{
-    m_xml.clear();
-    m_xml.addData(data);
-}
 
-std::unique_ptr<AlertEntry> CAPParser::parse()
+CAPAlertMessage CAPParser::parse()
 {
-    auto entry = std::make_unique<AlertEntry>();
+    CAPAlertMessage entry;
     while (m_xml.readNextStartElement()) {
         const auto tag = stringToValue(m_xml.name(), tag_map);
         if (!tag) {
@@ -154,19 +175,19 @@ std::unique_ptr<AlertEntry> CAPParser::parse()
         }
         switch (*tag) {
         case Tags::IDENTIFIER:
-            entry->setIdentifier(m_xml.readElementText());
+            entry.setIdentifier(m_xml.readElementText());
             break;
         case Tags::SENDER:
-            entry->setSender(m_xml.readElementText());
+            entry.setSender(m_xml.readElementText());
             break;
         case Tags::SENT_TIME:
-            entry->setSentTime(QDateTime::fromString(m_xml.readElementText(), Qt::ISODate));
+            entry.setSentTime(QDateTime::fromString(m_xml.readElementText(), Qt::ISODate));
             break;
         case Tags::STATUS: {
             const auto elementText = m_xml.readElementText();
             const auto status = stringToValue(elementText, status_map);
             if (status) {
-                entry->setStatus(*status);
+                entry.setStatus(*status);
             } else {
                 qWarning() << "Unknown status field" << elementText;
             }
@@ -176,7 +197,7 @@ std::unique_ptr<AlertEntry> CAPParser::parse()
             const auto elementText = m_xml.readElementText();
             const auto msgType = stringToValue(elementText, msgtype_map);
             if (msgType) {
-                entry->setMsgType(*msgType);
+                entry.setMessageType(*msgType);
             } else {
                 qWarning() << "Unknown msgType field" << elementText;
             }
@@ -186,20 +207,23 @@ std::unique_ptr<AlertEntry> CAPParser::parse()
             const auto elementText = m_xml.readElementText();
             const auto scope = stringToValue(elementText, scope_map);
             if (scope) {
-                entry->setScope(*scope);
+                entry.setScope(*scope);
             } else {
                 qWarning() << "Unknown scope field" << elementText;
             }
             break;
         }
         case Tags::NOTE:
-            entry->setNote(m_xml.readElementText());
+            entry.setNote(m_xml.readElementText());
             break;
         case Tags::INFO: {
             auto info = parseInfo();
-            entry->addInfo(info);
+            entry.addInfo(std::move(info));
             break;
         }
+        case Tags::REFERENCES:
+            entry.setReferences(parseReferences(m_xml.readElementText()));
+            break;
         default:
             m_xml.skipCurrentElement();
         }
@@ -207,9 +231,9 @@ std::unique_ptr<AlertEntry> CAPParser::parse()
     return entry;
 }
 
-AlertInfo CAPParser::parseInfo()
+CAPAlertInfo CAPParser::parseInfo()
 {
-    AlertInfo info;
+    CAPAlertInfo info;
 
     if (m_xml.name() == QLatin1String("info")) {
         while (!m_xml.atEnd() && !(m_xml.isEndElement() && m_xml.name() == QLatin1String("info"))) {
@@ -219,7 +243,6 @@ AlertInfo CAPParser::parseInfo()
             }
             const auto tag = stringToValue(m_xml.name(), info_tag_map);
             if (tag) {
-                qDebug() << m_xml.name();
                 switch (*tag) {
                 case InfoTags::CATEGORY: {
                     const auto s = m_xml.readElementText();
@@ -260,38 +283,11 @@ AlertInfo CAPParser::parseInfo()
                     info.setInstruction(m_xml.readElementText());
                     break;
                 case InfoTags::PARAMETER: {
-                    std::pair<QString, QString> p;
-                    m_xml.readNextStartElement();
-                    if (m_xml.name() == QStringLiteral("valueName")) {
-                        p.first = m_xml.readElementText();
-                    }
-                    m_xml.readNextStartElement();
-                    if (m_xml.name() == QStringLiteral("value")) {
-                        p.second = m_xml.readElementText();
-                    }
-                    info.addParameter(p);
+                    info.addParameter(parseNamedValue());
                     break;
                 }
                 case InfoTags::AREA: {
-                    while (!(m_xml.isEndElement() && m_xml.name() == QStringLiteral("area"))) {
-                        if (m_xml.name() == QStringLiteral("areaDesc") && !m_xml.isEndElement()) {
-                            info.setAreaDesc(m_xml.readElementText());
-                        } else if (m_xml.name() == QStringLiteral("geocode") && !m_xml.isEndElement()) {
-                            std::pair<QString, QString> p;
-                            m_xml.readNextStartElement();
-                            if (m_xml.name() == QStringLiteral("valueName")) {
-                                p.first = m_xml.readElementText();
-                            }
-                            m_xml.readNextStartElement();
-                            if (m_xml.name() == QStringLiteral("value")) {
-                                p.second = m_xml.readElementText();
-                            }
-                            info.addAreaCode(p);
-                        } else if (m_xml.name() == QStringLiteral("polygon") && !m_xml.isEndElement()) {
-                            info.addPolygon(KWeatherCorePrivate::stringToPolygon(m_xml.readElementText()));
-                        }
-                        m_xml.readNext();
-                    }
+                    info.addArea(parseArea());
                     break;
                 }
                 case InfoTags::SENDERNAME: {
@@ -301,6 +297,24 @@ AlertInfo CAPParser::parseInfo()
                 case InfoTags::LANGUAGE:
                     info.setLanguage(m_xml.readElementText());
                     break;
+                case InfoTags::RESPONSETYPE: {
+                    const auto elementText = m_xml.readElementText();
+                    if (const auto respType = stringToValue(elementText, response_type_map)) {
+                        info.addResponseType(*respType);
+                    } else {
+                        qWarning() << "Unknown respone type value" << elementText;
+                    }
+                    break;
+                }
+                case InfoTags::CONTACT:
+                    info.setContact(m_xml.readElementText());
+                    break;
+                case InfoTags::WEB:
+                    info.setWeb(m_xml.readElementText());
+                    break;
+                case InfoTags::EVENTCODE:
+                    info.addEventCode(parseNamedValue());
+                    break;
                 }
             } else {
                 if (m_xml.isStartElement()) {
@@ -311,4 +325,72 @@ AlertInfo CAPParser::parseInfo()
     }
     return info;
 }
+
+CAPArea CAPParser::parseArea()
+{
+    CAPArea area;
+    while (!(m_xml.isEndElement() && m_xml.name() == QStringLiteral("area"))) {
+        m_xml.readNext();
+        if (m_xml.name() == QStringLiteral("areaDesc") && !m_xml.isEndElement()) {
+            area.setDescription(m_xml.readElementText());
+        } else if (m_xml.name() == QStringLiteral("geocode") && !m_xml.isEndElement()) {
+            area.addGeoCode(parseNamedValue());
+        } else if (m_xml.name() == QStringLiteral("polygon") && !m_xml.isEndElement()) {
+            area.addPolygon(KWeatherCorePrivate::stringToPolygon(m_xml.readElementText()));
+        } else if (m_xml.name() == QLatin1String("circle") && !m_xml.isEndElement()) {
+            const auto t = m_xml.readElementText();
+            const auto commaIdx = t.indexOf(QLatin1Char(','));
+            const auto spaceIdx = t.indexOf(QLatin1Char(' '));
+            if (commaIdx > 0 && spaceIdx > commaIdx && commaIdx < t.size()) {
+                CAPCircle circle;
+                circle.latitude = QStringView(t).left(commaIdx).toFloat();
+                circle.longitude = QStringView(t).mid(commaIdx + 1, spaceIdx - commaIdx - 1).toFloat();
+                circle.radius = QStringView(t).mid(spaceIdx).toFloat();
+                area.addCircle(std::move(circle));
+            }
+        } else if (m_xml.name() == QLatin1String("altitude") && !m_xml.isEndElement()) {
+            area.setAltitude(m_xml.readElementText().toFloat());
+        } else if (m_xml.name() == QLatin1String("ceiling") && !m_xml.isEndElement()) {
+            area.setCeiling(m_xml.readElementText().toFloat());
+        } else if (m_xml.isStartElement()) {
+            qDebug() << "unknown area element:" << m_xml.name();
+        }
+    }
+    return area;
+}
+
+CAPNamedValue CAPParser::parseNamedValue()
+{
+    CAPNamedValue value;
+    const auto elementName = m_xml.name().toString();
+    while (!m_xml.isEndElement() || m_xml.name() != elementName) {
+        m_xml.readNext();
+        if (m_xml.isStartElement() && m_xml.name() == QLatin1String("valueName")) {
+            value.name = m_xml.readElementText();
+        } else if (m_xml.isStartElement() && m_xml.name() == QLatin1String("value")) {
+            value.value = m_xml.readElementText();
+        } else if (m_xml.isStartElement()) {
+            qDebug() << "unknown named value element:" << m_xml.name();
+        }
+    }
+    return value;
+}
+
+std::vector<CAPReference> CAPParser::parseReferences(const QString &refsString)
+{
+    std::vector<CAPReference> refs;
+    // TODO for Qt 6: use QStringTokenizer
+    const auto refsSplit = refsString.split(QLatin1Char(' '), Qt::SkipEmptyParts);
+    refs.reserve(refsSplit.size());
+    for (const auto &refString : refsSplit) {
+        const auto refSplit = refString.split(QLatin1Char(','));
+        if (refSplit.size() != 3) {
+            qDebug() << "failed to parse CAP reference:" << refString;
+            continue;
+        }
+        refs.emplace_back(refSplit.at(0), refSplit.at(1), QDateTime::fromString(refSplit.at(2), Qt::ISODate));
+    }
+
+    return refs;
+}
 }
