@@ -25,7 +25,6 @@ public:
     QString weatherDescription = QStringLiteral("Unknown");
     QDate date;
 
-    Sunrise sunrise;
     std::vector<HourlyWeatherForecast> hourlyWeatherForecast;
 };
 DailyWeatherForecast::~DailyWeatherForecast() = default;
@@ -64,7 +63,6 @@ QJsonObject DailyWeatherForecast::toJson()
         hourlyArray.append(h.toJson());
     }
     obj[QStringLiteral("hourly")] = hourlyArray;
-    obj[QStringLiteral("sunrise")] = sunrise().toJson();
     return obj;
 }
 DailyWeatherForecast DailyWeatherForecast::fromJson(QJsonObject obj)
@@ -78,7 +76,6 @@ DailyWeatherForecast DailyWeatherForecast::fromJson(QJsonObject obj)
     ret.setPressure(obj[QStringLiteral("pressure")].toDouble());
     ret.setWeatherIcon(obj[QStringLiteral("weatherIcon")].toString());
     ret.setWeatherDescription(obj[QStringLiteral("weatherDescription")].toString());
-    ret.setSunrise(Sunrise::fromJson(obj[QStringLiteral("sunrise")].toObject()));
     std::vector<HourlyWeatherForecast> hourlyVec;
     auto array = obj[QStringLiteral("hourly")].toArray();
     for (int i = 0; i < array.size(); i++) {
@@ -171,18 +168,10 @@ QDateTime DailyWeatherForecast::dateTime() const
 {
     return d->date.startOfDay();
 }
-const Sunrise &DailyWeatherForecast::sunrise() const
-{
-    return d->sunrise;
-}
 const std::vector<HourlyWeatherForecast> &DailyWeatherForecast::hourlyWeatherForecast() const
 {
     return d->hourlyWeatherForecast;
 }
-void DailyWeatherForecast::setSunrise(Sunrise sunrise)
-{
-    d->sunrise = std::move(sunrise);
-}
 void DailyWeatherForecast::setHourlyWeatherForecast(const std::vector<HourlyWeatherForecast> &forecast)
 {
     d->hourlyWeatherForecast = forecast;
@@ -191,31 +180,6 @@ void DailyWeatherForecast::setHourlyWeatherForecast(std::vector<HourlyWeatherFor
 {
     d->hourlyWeatherForecast = std::move(forecast);
 }
-DailyWeatherForecast &DailyWeatherForecast::operator+(const DailyWeatherForecast &forecast)
-{
-    if (date().isNull()) {
-        setDate(forecast.date());
-        setWeatherDescription(forecast.weatherDescription());
-        setWeatherIcon(forecast.weatherIcon());
-        d->isValid = false;
-    }
-
-    if (*this == forecast) {
-        setPrecipitation(precipitation() + forecast.precipitation());
-        setUvIndex(std::max(uvIndex(), forecast.uvIndex()));
-        setHumidity(std::max(humidity(), forecast.humidity()));
-        setPressure(std::max(pressure(), forecast.pressure()));
-        setMaxTemp(std::max(maxTemp(), forecast.maxTemp()));
-        setMinTemp(std::min(minTemp(), forecast.minTemp()));
-    }
-
-    return *this;
-}
-
-DailyWeatherForecast &DailyWeatherForecast::operator+=(const DailyWeatherForecast &forecast)
-{
-    return *this + forecast;
-}
 
 DailyWeatherForecast &DailyWeatherForecast::operator+=(const HourlyWeatherForecast &forecast)
 {
